@@ -32,16 +32,22 @@ class CausalGraph:
         self.redundant_nodes = []  # store the list of redundant nodes (for subgraphs)
         self.nx_graph = nx.DiGraph()  # store the directed graph
         self.nx_skel = nx.Graph()  # store the undirected graph
+        self.prt_m = {}  # store the parents of missingness indicators
+        self.mvpc = None
 
 
-    def set_ind_test(self, indep_test):
+    def set_ind_test(self, indep_test, mvpc = False):
         """Set the conditional independence test that will be used"""
         # assert name_of_test in ["Fisher_Z", "Chi_sq", "G_sq"]
+        if mvpc:
+            self.mvpc = True
         self.test = indep_test
 
 
     def ci_test(self, i, j, S):
         """Define the conditional independence test"""
+        if self.mvpc:
+            return self.test(self.data, self.nx_skel, self.prt_m, i, j, S, self.data.shape[0])
         return self.test(self.data, i, j, S)
 
 
