@@ -164,7 +164,7 @@ def ges(X, score_func, maxP=None, parameters=None):
                 break
             G = insert(G, min_desc[0], min_desc[1], min_desc[2])
             update1.append([min_desc[0], min_desc[1], min_desc[2]])
-            print(G.graph)
+            # print(G.graph)
             G = pdag2dag(G)
             G = dag2cpdag(G)
             G_step1.append(G)
@@ -228,7 +228,7 @@ def ges(X, score_func, maxP=None, parameters=None):
                 break
             G = delete(G, min_desc[0], min_desc[1], min_desc[2])
             update2.append([min_desc[0], min_desc[1], min_desc[2]])
-            print(G.graph)
+            # print(G.graph)
             G = pdag2dag(G)
             G = dag2cpdag(G)
             G_step2.append(G)
