@@ -5,7 +5,7 @@ from pytrad.graph.Edges import Edges
 from pytrad.graph.Endpoint import Endpoint
 
 
-def uc_sepset(cg, priority=3):
+def uc_sepset(cg, priority=3, background_knowledge=None):
     '''
     Run (UC_sepset) to orient unshielded colliders
 
@@ -33,6 +33,12 @@ def uc_sepset(cg, priority=3):
     UT = [(i, j, k) for (i, j, k) in cg_new.find_unshielded_triples() if i < k]  # Not considering symmetric triples
 
     for (x, y, z) in UT:
+        if (background_knowledge is not None) and \
+                (background_knowledge.is_forbidden(cg_new.G.nodes[x], cg_new.G.nodes[y]) or
+                 background_knowledge.is_forbidden(cg_new.G.nodes[z], cg_new.G.nodes[y]) or
+                 background_knowledge.is_required(cg_new.G.nodes[y], cg_new.G.nodes[x]) or
+                 background_knowledge.is_required(cg_new.G.nodes[y], cg_new.G.nodes[z])):
+            continue
         if all(y not in S for S in cg.sepset[x, z]):
             if priority == 0:  # 0: overwrite
                 edge1 = cg_new.G.get_edge(cg_new.G.nodes[x], cg_new.G.nodes[y])
@@ -108,6 +114,12 @@ def uc_sepset(cg, priority=3):
             UC_dict = sort_dict_ascending(UC_dict, descending=True)
 
         for (x, y, z) in UC_dict.keys():
+            if (background_knowledge is not None) and \
+                    (background_knowledge.is_forbidden(cg_new.G.nodes[x], cg_new.G.nodes[y]) or
+                     background_knowledge.is_forbidden(cg_new.G.nodes[z], cg_new.G.nodes[y]) or
+                     background_knowledge.is_required(cg_new.G.nodes[y], cg_new.G.nodes[x]) or
+                     background_knowledge.is_required(cg_new.G.nodes[y], cg_new.G.nodes[z])):
+                continue
             if (not cg_new.is_fully_directed(y, x)) and (not cg_new.is_fully_directed(y, z)):
                 edge1 = cg_new.G.get_edge(cg_new.G.nodes[x], cg_new.G.nodes[y])
                 if edge1 is not None:
@@ -124,7 +136,7 @@ def uc_sepset(cg, priority=3):
 
 
 
-def maxp(cg, priority=3):
+def maxp(cg, priority=3, background_knowledge=None):
     '''
     Run (MaxP) to orient unshielded colliders
 
@@ -150,6 +162,13 @@ def maxp(cg, priority=3):
     UT = [(i, j, k) for (i, j, k) in cg_new.find_unshielded_triples() if i < k]  # Not considering symmetric triples
 
     for (x, y, z) in UT:
+        if (background_knowledge is not None) and \
+                (background_knowledge.is_forbidden(cg_new.G.nodes[x], cg_new.G.nodes[y]) or
+                 background_knowledge.is_forbidden(cg_new.G.nodes[z], cg_new.G.nodes[y]) or
+                 background_knowledge.is_required(cg_new.G.nodes[y], cg_new.G.nodes[x]) or
+                 background_knowledge.is_required(cg_new.G.nodes[y], cg_new.G.nodes[z])):
+            continue
+
         cond_with_y = cg_new.find_cond_sets_with_mid(x, z, y)
         cond_without_y = cg_new.find_cond_sets_without_mid(x, z, y)
 
@@ -227,6 +246,13 @@ def maxp(cg, priority=3):
             UC_dict = sort_dict_ascending(UC_dict, True)
 
         for (x, y, z) in UC_dict.keys():
+            if (background_knowledge is not None) and \
+                    (background_knowledge.is_forbidden(cg_new.G.nodes[x], cg_new.G.nodes[y]) or
+                     background_knowledge.is_forbidden(cg_new.G.nodes[z], cg_new.G.nodes[y]) or
+                     background_knowledge.is_required(cg_new.G.nodes[y], cg_new.G.nodes[x]) or
+                     background_knowledge.is_required(cg_new.G.nodes[y], cg_new.G.nodes[z])):
+                continue
+
             if (not cg_new.is_fully_directed(y, x)) and (not cg_new.is_fully_directed(y, z)):
                 edge1 = cg_new.G.get_edge(cg_new.G.nodes[x], cg_new.G.nodes[y])
                 if edge1 is not None:
@@ -241,7 +267,7 @@ def maxp(cg, priority=3):
 
         return cg_new
 
-def definite_maxp(cg, alpha, priority=4):
+def definite_maxp(cg, alpha, priority=4, background_knowledge=None):
     '''
     Run (Definite_MaxP) to orient unshielded colliders
 
@@ -316,6 +342,13 @@ def definite_maxp(cg, alpha, priority=4):
         UC_dict = sort_dict_ascending(UC_dict, True)
 
     for (x, y, z) in UC_dict.keys():
+        if (background_knowledge is not None) and \
+                (background_knowledge.is_forbidden(cg_new.G.nodes[x], cg_new.G.nodes[y]) or
+                 background_knowledge.is_forbidden(cg_new.G.nodes[z], cg_new.G.nodes[y]) or
+                 background_knowledge.is_required(cg_new.G.nodes[y], cg_new.G.nodes[x]) or
+                 background_knowledge.is_required(cg_new.G.nodes[y], cg_new.G.nodes[z])):
+            continue
+
         if (not cg_new.is_fully_directed(y, x)) and (not cg_new.is_fully_directed(y, z)):
             edge1 = cg_new.G.get_edge(cg_new.G.nodes[x], cg_new.G.nodes[y])
             if edge1 is not None:
